import requests
import pprint
import base64  # Required for manual creation of authKey
import json

pp = pprint.PrettyPrinter(indent=4)

# Date: 05/21/2020
# Author: Felix Yang
# Description: Program that demos sample API calls
# Requirements: requests package
# Notes: All the tests done below are made using the test API
#        Replace the baseURL with https://sign.signority.com/api
#        For actual use

# Change to your username and your password, APIKey is not needed at this time
username = "signorityapidemo@gmail.com"
password = "password"
apiKey = ""
authKey = ""

'''
__________________________________________________________
1. Login
    Logging in using username and password
__________________________________________________________
'''

# Login using Password and Username
login_response = requests.get(
    "https://test.signority.com/api/v1/user/login?email=" + username + "&password=" + password)

# Sets APIKey to the logged in User's API Key
apiKey = login_response.json()['user']['userSetting']['APIKey']

# Manual generation of Authentication Key
# bytes = (apiKey + ":").encode('ascii')
# b64bytes = base64.b64encode(bytes)
# authKey1 = "Basic " + b64bytes.decode('ascii')
# print(authKey1)

# Sets authKey to the logged in user's Authorization key (If this doesn't work, use the above manual method)
authKey = login_response.json()['user']['userSetting']['authKey']

# Prints out the response JSON with pprint formatting
pp.pprint(login_response.json())

'''
__________________________________________________________
2. Account Details
    After being logged in, you can access the logged in
    user's information
__________________________________________________________
'''

detail_response = requests.get("https://test.signority.com/api/v1/user/", auth=(apiKey, ""))
pp.pprint(detail_response.json())

'''
__________________________________________________________
3. Create a new Document/Template
    See documentation for information regarding the data
    types expected in each field
__________________________________________________________
'''

# Put all of the necessary information in the JSON to create a new doc, see documentation for more help
cdoc_JSON = json.dumps({
    "documents": [
        {
            "title": "API Test document 001",
            "senderName": "Python User 27",
            "senderEmail": "pythonuser27@mail.com",
            "signingStatus": 51,
            "documentType": 8,
            "invitations": [
                {
                    "id": -1,
                    "recipientName": "Python Recipient",
                    "recipientEmail": "pythonrecip@mail.com",
                    "actionType": 15,
                    "status": 60,
                    "sequence": 1
                }
            ]
        }
    ]
})
cdoc_response = requests.post("https://test.signority.com/api/v1/documents/array", data=cdoc_JSON, auth=(apiKey, ""))

# At this point, if you log in and check your documents, A new document will have been created
pp.pprint(cdoc_response.json())

# Storing the Document ID for future use
doc_id = cdoc_response.json()['documents'][0]['id']

'''
__________________________________________________________
4. Uploading files to a document
    See documentation for information regarding the data
    types expected in each field
__________________________________________________________
'''

url = 'https://test.signority.com/api/v1/documents/' + str(doc_id) + '/files.html'

# Set the JSON with the required tags + Files that you would like to upload
upload_JSON = {
    "addTagsOfPDF": False,
    "addAnchorTags": False,
    "highDisplayResolution": False,
}

files = [
    ('files', open('blank.pdf', 'rb')),
    ('files', open('blank.pdf', 'rb'))
]

upload_response = requests.post(url, files=files, data=upload_JSON, auth=(apiKey, ''))

# file_id stores the id of the first file
file_id = upload_response.json()['files'][0]['id']
pp.pprint(upload_response.json())

'''
__________________________________________________________
5. Updating + Sending a Document
    Note: When the file is being updated, it must be sent 
    to the invitees after it's done
    Note: After the document is sent, modifications
    to it's contents can longer be made
__________________________________________________________
'''

# The example JSON just sends the documents to the invitees
update_JSON = json.dumps({
    "document": {
        "signingStatus": 51
    }
})

update_response = requests.put('https://test.signority.com/api/v1/documents/' + str(doc_id), update_JSON,
                               auth=(apiKey, ''))
pp.pprint(update_response.json())

'''
_____________________________________________________________
6. Update Document Tags
    Not recommended way to add tags to a document but can
    be done, the better way is to use Anchor Tags.
    Anchor Tags:
    https://www.signority.com/knowledge-base/use-anchor-text/
_____________________________________________________________
'''

uDoc_JSON = json.dumps({
        "tags": [
            {
                "pageIndex": 0,
                "invitationIndex": 0,
                "name": "U01",
                "isMandatory": True,
                "height": 35,
                "width": 107,
                "left": 200,
                "top": 200,
                "type": 250
            },
            {
                "pageIndex": 0,
                "invitationIndex": 0,
                "name": "U02",
                "isMandatory": True,
                "height": 35,
                "width": 107,
                "left": 200,
                "top": 200,
                "type": 250
            }
        ]
})

uDoc_response = requests.put('https://test.signority.com/api/v1/documents/' + str(doc_id) + '/tags/', uDoc_JSON,
                             auth=(apiKey, ''))
pp.pprint(uDoc_response.json())

'''
__________________________________________________________
7. Bulk Sign using a Template
    Sends the template to many signatures in the
    invitation list
__________________________________________________________
'''

bulk_JSON = json.dumps({
  "callback": "",
  "invitations": [
    {
      "recipientName": "API TEST USER01",
      "recipientEmail": "postman01@mail.com",
      "user_name": "Postman User 1",
      "user_tel": "1234567890",
      "user_add": "780-2300 St Laurent Street",
      "user_code": "ST20200521"
    },
    {
      "recipientName": "API TEST USER02",
      "recipientEmail": "postman02@mail.com",
      "user_name": "Postman User 2",
      "user_tel": "0987654321",
      "user_add": "780-2300 St Laurent Street",
      "user_code": "ST20200522"
    }
  ]})

# Note that this only works for bulk sign templates
#bulk_response = requests.post('https://test.signority.com/api/v1/template/use/' + str(doc_id), bulk_JSON, auth=(apiKey, ''))
#pp.pprint(bulk_response.json())

'''
__________________________________________________________
8. Retrieve Document Data
    On success it will return the information on the
    queried document
__________________________________________________________
'''

retr_response = requests.get('https://test.signority.com/api/v1/documents/' + str(doc_id), auth=(apiKey, ''))
pp.pprint(retr_response.json())

'''
__________________________________________________________
9. Get a List of Documents
    Returns all of the documents that is owned or
    accessible to the logged in user
__________________________________________________________
'''

gDoc_response = requests.get('https://test.signority.com/api/v2/documents/search2', auth=(apiKey, ''))
pp.pprint(gDoc_response.json())

'''
__________________________________________________________
10. Update Invitation List
    Change all of the invitees on a document/template
__________________________________________________________
'''

uInv_JSON = json.dumps({
    "invitations": [
        {
            "sequence": 1,
            "recipientName": "API TEST USER01",
            "recipientEmail": "user01@mail.com",
            "actionType": 15
        },
        {
            "sequence": 2,
            "recipientName": "API TEST USER02",
            "recipientEmail": "user02@mail.com",
            "actionType": 15
        }
    ]
})

uInt_response = requests.put('https://test.signority.com/api/v1/documents/' + str(doc_id) + '/invitations/', uInv_JSON,
                             auth=(apiKey, ''))
pp.pprint(uInt_response.json())

'''
__________________________________________________________
11. Get a List of Invitees
    Returns all of the invitees on a given document that
    the signed in user has access to
__________________________________________________________
'''

gInv_response = requests.get('https://test.signority.com/api/v1/documents/' + str(doc_id) + '/invitations/',
                             auth=(apiKey, ''))
pp.pprint(gInv_response.json())

'''
__________________________________________________________
12. Get List of Files
    Returns all of the files in a document
__________________________________________________________
'''

gFiles_response = requests.get('https://test.signority.com/api/v1/documents/' + str(doc_id) + '/files/',
                               auth=(apiKey, ''))
pp.pprint(gFiles_response.json())

'''
__________________________________________________________
13. Delete a File
    Deleted a file from the document
__________________________________________________________
'''

del_response = requests.delete('https://test.signority.com/api/v1/documents/' + str(doc_id) + '/files/' + str(file_id),
                               auth=(apiKey, ''))
pp.pprint(del_response.json())
